package main

import (
	"encoding/json"
	"errors"
	"net/url"
	"strconv"
)

var ApiNeedsRouting = map[string]DispatchFunction{
	"GET":    listNeeds,
	"PUT":    addNeed,
	"DELETE": delNeed,
	"PATCH":  editNeed,
}

func listNeeds(args []string, body []byte) ([]byte, error) {
	if len(args) < 1 {
		if needs, err := GetNeeds(); err != nil {
			return nil, err
		} else if ret, err := json.Marshal(needs); err != nil {
			return nil, err
		} else {
			return ret, nil
		}
	} else if i, err := strconv.ParseInt(args[0], 10, 64); err != nil {
		return nil, err
	} else {
		if need, err := GetNeed(i); err != nil {
			return nil, err
		} else if ret, err := json.Marshal(need); err != nil {
			return nil, err
		} else {
			return ret, nil
		}
	}
}

func addNeed(args []string, body []byte) ([]byte, error) {
	if m, err := url.ParseQuery(string(body)); err != nil {
		return nil, err
	} else if idp := m.Get("product"); idp == "" {
		return nil, errors.New("No product identifier given")
	} else if id_product, err := strconv.ParseInt(idp, 10, 64); err != nil {
		return nil, err
	} else if product, err := GetProduct(id_product); err != nil {
		return nil, err
	} else if q := m.Get("quantity"); q == "" {
		return nil, errors.New("No quantity given")
	} else if quantity, err := strconv.ParseInt(q, 10, 64); err != nil {
		return nil, err
	} else if id, err := CreateNeed(product, quantity); err != nil {
		return nil, err
	} else if r, err := json.Marshal(map[string]int64{"id": id}); err != nil {
		return nil, err
	} else {
		return r, nil
	}
}

func delNeed(args []string, body []byte) ([]byte, error) {
	if len(args) < 1 {
		return nil, errors.New("Need a need identifier to delete")
	} else if id, err := strconv.ParseInt(args[0], 10, 64); err != nil {
		return nil, err
	} else if need, err := GetNeed(id); err != nil {
		return nil, err
	} else if err := need.Delete(); err != nil {
		return nil, err
	} else if r, err := json.Marshal(map[string]int64{"id": id}); err != nil {
		return nil, err
	} else {
		return r, nil
	}
}

func editNeed(args []string, body []byte) ([]byte, error) {
	if len(args) < 1 {
		return nil, errors.New("Need a need identifier to patch")
	} else if id, err := strconv.ParseInt(args[0], 10, 64); err != nil {
		return nil, err
	} else if need, err := GetNeed(id); err != nil {
		return nil, err
	} else if m, err := url.ParseQuery(string(body)); err != nil {
		return nil, err
	} else if idp, err := strconv.ParseInt(m.Get("product"), 10, 64); err != nil {
		return nil, err
	} else if nproduct, err := GetProduct(idp); err != nil {
		return nil, err
	} else if quantity, err := strconv.ParseInt(m.Get("quantity"), 10, 64); err != nil {
		return nil, err
	} else {
		need.SetProduct(nproduct)
		need.Quantity = quantity

		if err := need.Save(); err != nil {
			return nil, err
		} else {
			return nil, nil
		}
	}
}
