package main

import (
	"encoding/json"
	"errors"
	"net/url"
	"strconv"
)

var ApiPurchasesRouting = map[string]DispatchFunction{
	"GET":    listPurchases,
	"PUT":    addPurchase,
	"DELETE": delPurchase,
	"PATCH":  editPurchase,
}

func listPurchases(args []string, body []byte) ([]byte, error) {
	if len(args) < 1 {
		if purchases, err := GetPurchases(); err != nil {
			return nil, err
		} else if ret, err := json.Marshal(purchases); err != nil {
			return nil, err
		} else {
			return ret, nil
		}
	} else if i, err := strconv.ParseInt(args[0], 10, 64); err != nil {
		return nil, err
	} else {
		if purchase, err := GetPurchase(i); err != nil {
			return nil, err
		} else if ret, err := json.Marshal(purchase); err != nil {
			return nil, err
		} else {
			return ret, nil
		}
	}
}

func addPurchase(args []string, body []byte) ([]byte, error) {
	if m, err := url.ParseQuery(string(body)); err != nil {
		return nil, err
	} else if idp := m.Get("product"); idp == "" {
		return nil, errors.New("No product identifier given")
	} else if id_product, err := strconv.ParseInt(idp, 10, 64); err != nil {
		return nil, err
	} else if product, err := GetProduct(id_product); err != nil {
		return nil, err
	} else if q := m.Get("quantity"); q == "" {
		return nil, errors.New("No quantity given")
	} else if quantity, err := strconv.ParseInt(q, 10, 64); err != nil {
		return nil, err
	} else if date := m.Get("date"); date == "" {
		return nil, errors.New("No date given")
	} else if id, err := CreatePurchase(product, date, quantity); err != nil {
		return nil, err
	} else if r, err := json.Marshal(map[string]int64{"id": id}); err != nil {
		return nil, err
	} else {
		return r, nil
	}
}

func delPurchase(args []string, body []byte) ([]byte, error) {
	if len(args) < 1 {
		return nil, errors.New("Need a purchase identifier to delete")
	} else if id, err := strconv.ParseInt(args[0], 10, 64); err != nil {
		return nil, err
	} else if purchase, err := GetPurchase(id); err != nil {
		return nil, err
	} else if err := purchase.Delete(); err != nil {
		return nil, err
	} else if r, err := json.Marshal(map[string]int64{"id": id}); err != nil {
		return nil, err
	} else {
		return r, nil
	}
}

func editPurchase(args []string, body []byte) ([]byte, error) {
	if len(args) < 1 {
		return nil, errors.New("Need a purchase identifier to patch")
	} else if id, err := strconv.ParseInt(args[0], 10, 64); err != nil {
		return nil, err
	} else if purchase, err := GetPurchase(id); err != nil {
		return nil, err
	} else if m, err := url.ParseQuery(string(body)); err != nil {
		return nil, err
	} else if idp, err := strconv.ParseInt(m.Get("product"), 10, 64); err != nil {
		return nil, err
	} else if nproduct, err := GetProduct(idp); err != nil {
		return nil, err
	} else if date := m.Get("date"); date == "" {
		return nil, errors.New("No date given")
	} else if quantity, err := strconv.ParseInt(m.Get("quantity"), 10, 64); err != nil {
		return nil, err
	} else {
		purchase.SetProduct(nproduct)
		purchase.Date = date
		purchase.Quantity = quantity

		if err := purchase.Save(); err != nil {
			return nil, err
		} else {
			return nil, nil
		}
	}
}
