package main

import (
	"flag"
	"fmt"
	"log"
	"net/http"
	"os"
)

func main() {
	var port = flag.Int("port", 8080, "Listening port")
	flag.Parse()

	log.Println("Opening database...")
	if err := DBInit("courses.db"); err != nil {
		log.Fatal("Cannot open the database: ", err)
		os.Exit(1)
	}
	defer DBClose()

	log.Println("Creating database...")
	if err := DBCreate(); err != nil {
		log.Fatal("Cannot create database: ", err)
		os.Exit(1)
	}

	log.Println("Registering handlers...")
	mux := http.NewServeMux()
	mux.Handle("/favicon.ico", http.FileServer(http.Dir("./static/")))
	mux.Handle("/static/", http.FileServer(http.Dir("./")))
	mux.HandleFunc("/api/", ApiRouting)
	http.HandleFunc("/", mux.ServeHTTP)

	log.Println(fmt.Sprintf("Ready, listening on port %d", *port))
	http.ListenAndServe(fmt.Sprintf(":%d", *port), nil)
}
