package main

type Need struct {
	id        int64
	idProduct int64
	Quantity  int64
}

func GetNeeds() ([]Need, error) {
	if rows, err := DBQuery("SELECT id_need, id_product, quantity FROM needs ORDER BY id_need"); err != nil {
		return nil, err
	} else {
		defer rows.Close()

		var needs = make([]Need, 0)
		for rows.Next() {
			var n Need
			if err := rows.Scan(&n.id, &n.idProduct, &n.Quantity); err != nil {
				return nil, err
			}
			needs = append(needs, n)
		}
		if err := rows.Err(); err != nil {
			return nil, err
		}

		return needs, nil
	}
}

func GetNeed(id int64) (Need, error) {
	var n Need
	if err := DBQueryRow("SELECT id_need, id_product, quantity FROM needs WHERE id_need=?", id).Scan(&n.id, &n.idProduct, &n.Quantity); err != nil {
		return n, err
	}
	return n, nil
}

func (this Need) Product() (Product, error) {
	return GetProduct(this.idProduct)
}

func (this Need) SetProduct(p Product) {
	this.idProduct = p.Id
}

func (this Need) Save() error {
	if _, err := DBExec("UPDATE needs SET id_product=?, quantity=? WHERE id_need=?", this.idProduct, this.Quantity, this.id); err != nil {
		return err
	} else {
		return nil
	}
}

func (this Need) Delete() error {
	if _, err := DBExec("DELETE FROM needs WHERE id_need=?", this.id); err != nil {
		return err
	} else {
		return nil
	}
}

func CreateNeed(p Product, quantity int64) (int64, error) {
	if res, err := DBExec("INSERT INTO needs (id_product, quantity) VALUES (?, ?)", p.Id, quantity); err != nil {
		return 0, err
	} else if id, err := res.LastInsertId(); err != nil {
		return 0, err
	} else {
		return id, nil
	}
}
