package main

type Product struct {
	Id   int64
	Name string
}

func GetProducts() ([]Product, error) {
	if rows, err := DBQuery("SELECT id_product, name FROM products ORDER BY id_product"); err != nil {
		return nil, err
	} else {
		defer rows.Close()

		var products = make([]Product, 0)
		for rows.Next() {
			var p Product
			if err := rows.Scan(&p.Id, &p.Name); err != nil {
				return nil, err
			}
			products = append(products, p)
		}
		if err := rows.Err(); err != nil {
			return nil, err
		}

		return products, nil
	}
}

func GetProduct(id int64) (Product, error) {
	var p Product
	if err := DBQueryRow("SELECT id_product, name FROM products WHERE id_product=?", id).Scan(&p.Id, &p.Name); err != nil {
		return p, err
	}
	return p, nil
}

func AddProduct(name string) (int64, error) {
	if res, err := DBExec("INSERT INTO products (name) VALUES (?)", name); err != nil {
		return 0, err
	} else if id, err := res.LastInsertId(); err != nil {
		return 0, err
	} else {
		return id, nil
	}
}

func RemoveProduct(id int64) (int64, error) {
	if res, err := DBExec("DELETE FROM products WHERE id_product=?", id); err != nil {
		return 0, err
	} else if nb, err := res.RowsAffected(); err != nil {
		return 0, err
	} else {
		return nb, err
	}
}

func UpdateProduct(id int64, name string) (int64, error) {
	if res, err := DBExec("UPDATE products SET name=? WHERE id_product=?", name, id); err != nil {
		return 0, err
	} else if nb, err := res.RowsAffected(); err != nil {
		return 0, err
	} else {
		return nb, nil
	}
}
