package main

import (
	"time"
)

type Purchase struct {
	id        int64
	idProduct int64
	Date      time.Time
	Quantity  int64
}

func GetPurchases() ([]Purchase, error) {
	if rows, err := DBQuery("SELECT id_purchase, id_product, date, quantity FROM purchases ORDER BY id_purchase"); err != nil {
		return nil, err
	} else {
		defer rows.Close()

		var purchases = make([]Purchase, 0)
		for rows.Next() {
			var p Purchase
			if err := rows.Scan(&p.id, &p.idProduct, &p.Date, &p.Quantity); err != nil {
				return nil, err
			}
			purchases = append(purchases, p)
		}
		if err := rows.Err(); err != nil {
			return nil, err
		}

		return purchases, nil
	}
}

func GetPurchase(id int64) (Purchase, error) {
	var p Purchase
	if err := DBQueryRow("SELECT id_purchase, id_product, date, quantity FROM purchases WHERE id_purchase=?", id).Scan(&p.id, &p.idProduct, &p.Date, &p.Quantity); err != nil {
		return p, err
	}
	return p, nil
}

func (this Purchase) Product() (Product, error) {
	return GetProduct(this.idProduct)
}

func (this Purchase) SetProduct(p Product) {
	this.idProduct = p.Id
}

func (this Purchase) Save() error {
	if _, err := DBExec("UPDATE purchases SET id_product=?, date=?, quantity=? WHERE id_purchase=?", this.idProduct, this.Date, this.Quantity, this.id); err != nil {
		return err
	} else {
		return nil
	}
}

func (this Purchase) Delete() error {
	if _, err := DBExec("DELETE FROM purchases WHERE id_purchase=?", this.id); err != nil {
		return err
	} else {
		return nil
	}
}

func CreatePurchase(p Product, date time.Time, quantity int64) (int64, error) {
	if res, err := DBExec("INSERT INTO purchases (id_product, date, quantity) VALUES (?, ?, ?)", p.Id, date, quantity); err != nil {
		return 0, err
	} else if id, err := res.LastInsertId(); err != nil {
		return 0, err
	} else {
		return id, nil
	}
}
